//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library, Example9 - Fast Page Flipping
//
// Date: 7th January, 2004
// Last Updated: 18th May 2013
// Author: Darren John Dwyer
// Copyright: (c) 2002-2013 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to partially optimize page flipping.
// It uses Application->OnIdle() instead of Timer1->OnTimer()
// It uses global TCanvas's, rather than creating and destroying
// each time a TCanvas is required.
//
// See later examples for increased frame rates.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma link "tddscaps"
#pragma link "tddsurfacedesc"
#pragma link "tdx_draw"
#pragma link "tdx_drawsurface"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString IntToHexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
AnsiString string = "0x";
int value = 0x10000000;
for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Top=0;
Left=0;
Width=800;
Height=600;

canvas = new TCanvas();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
Application->OnIdle = NULL;
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
if (canvas) delete canvas;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
NumModes=0;
ModesListBox->Items->Clear();
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);

Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE );

// create a complex primary surface
PrimarySurfaceDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
PrimarySurfaceDesc->BackBufferCount = 1;
PrimaryCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_COMPLEX | DDSCAPS_FLIP;
PrimarySurfaceDesc->SCaps = PrimaryCaps;
PrimarySurface->Create( PrimarySurfaceDesc, Dx_Draw1 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
PrimarySurface->Destroy();
Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL );
if (Dx_Draw1->RestoreDisplayMode())
    {
    Top=0;
    Left=0;
    Width=800;
    Height=600;
    Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
      TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<MAX_MODES)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;

    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------



void __fastcall TMainForm::PrimarySurfaceCreate(TObject *Sender)
{
Msg(">>> Primary Surface Created");

// get it's attached backbuffer
if (PrimarySurface->GetAttachedSurface( BackBufferCaps, BackBuffer ))
    // call the backbuffer::OnCreate() event manually.
    BackBuffer->OnCreate(this);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceDestroy(TObject *Sender)
{
Msg(">>> Primary Surface Destroy");
if (BackBuffer->Created) BackBuffer->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PrimarySurfaceSurfaceLost(TObject *Sender)
{
PrimarySurface->Restore();
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::RepaintButtonClick(TObject *Sender)
{
Application->OnIdle = NULL;
if (Dx_Draw1->Created) Dx_Draw1->FlipToGDISurface();

    int old_top = Top;
    Top=Screen->Height+1;
    Invalidate();
    Top=old_top;
    Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferCreate(TObject *Sender)
{
Msg(">>> BackBuffer Created");

Image2->AutoSize = false;
Image2->Stretch = true;
Image2->Width = Screen->Width;
Image2->Height = Screen->Height;
Image2->Canvas->StretchDraw( Image2->Canvas->ClipRect, Image1->Picture->Graphic );
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::BackBufferDestroy(TObject *Sender)
{
Msg(">>> BackBuffer Destroyed");
}
//---------------------------------------------------------------------------


void __fastcall TMainForm::CreateButtonClick(TObject *Sender)
{
Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::FlipButtonClick(TObject *Sender)
{
ShowMessage("Please Note:\n\
After the primary surface has been painted the contents of the current form\n\
will no longer be visible though it is still active. To repaint the form\n\
Press ALT-R (Restore) or to exit the application, press ALT-X (eXit)" );

CurrentPageNumber = 0;
Application->OnIdle = OnIdle;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OnIdle(TObject *Sender, bool& Done)
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// error-proof vcl code
HDC hdc;
if (BackBuffer->GetDC(&hdc))
    {
    try
        {
        // make sure to reassign the canvas so it paints in the right spot
        canvas->Handle = hdc;

        // paint the background star image
        canvas->CopyRect( canvas->ClipRect, Image2->Canvas, Image2->Canvas->ClipRect );

        // paint some text
        PaintText( 20, 20, "Arial", 20, 128, 255, 128, "Example9 - Faster Page Flipping" );
        PaintText( 100 + CurrentPageNumber % Screen->Width, 50, "Comic Sans MS", 50, 128 - CurrentPageNumber % 256, 128 + CurrentPageNumber % 256, CurrentPageNumber % 256, "TDx_Library v1.90" );
        PaintText( 100 + CurrentPageNumber % Screen->Width, 200 + CurrentPageNumber % Screen->Height, "Arial", 20, 128, 255, 128, "Frame# "+IntToStr(CurrentPageNumber) );

        // and remove our temporary canvas handle
        canvas->Handle = NULL;
        }
    catch (Exception &e)
        {
        Application->OnIdle = NULL;
        }

    BackBuffer->ReleaseDC(hdc);
    }

PrimarySurface->Flip( BackBuffer, DDFLIP_DONOTWAIT );

++CurrentPageNumber %= Screen->Width;

// continue immediately
Done = false;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::PaintText( int x, int y, AnsiString font, int fontsize, int red, int green, int blue, AnsiString text )
{
if (!BackBuffer->Created) return;
if (!PrimarySurface->Created) return;

// paint some text
canvas->Brush->Style = bsClear;
canvas->Pen->Style = psClear;
canvas->Font->Name = font;
canvas->Font->Style = TFontStyles();
canvas->Font->Size = fontsize;

canvas->Font->Color = (TColor) RGB(0,0,0);
canvas->TextOut( x+2,y+2, text );

canvas->Font->Color = (TColor) RGB(red,green,blue);
canvas->TextOut( x,y, text );
}
//---------------------------------------------------------------------------


