//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library,
//       ExampleC - Painting onto Surfaces
//
// Date: 1st January, 2004
// Last Updated: 18th May 2013
// Author: Darren John Dwyer
// Copyright: (c) 2002-2013 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to paint onto the backbuffer using (3) different
// methods :-
//
// (1) paint onto the surface using a VCL-style TCanvas
// (2) paint onto the surface using standard Win32 code (somewhat)
// (3) paint onto the surface directly using custom paint code
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
#include <Dialogs.hpp>
#include <Graphics.hpp>
#include "TDx_DrawPalette.h"
#include "tddbltfx.h"
#include "TDDBltFX.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label5;

    TMemo *InfoMemo;
    TMemo *StatusMemo;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;
    TDx_Draw *Dx_Draw1;
    TLabel *Label7;
    TLabel *CurrentModeLabel;
    TDx_DrawSurface *PrimarySurface;
    TButton *PaintWin32;
    TButton *RepaintButton;
    TDx_DrawSurface *BackBuffer;
    TBitBtn *PaintVCL;
    TLabel *SelectedLabel;
    TOpenDialog *OpenDialog1;
    TLabel *Label4;
    TDDBltFX *DDBltFX1;
    TBitBtn *SnapshotButton;
    TLabel *CreatedLabel;
    TDDSurfaceDesc *PrimaryDesc;
    TDDSCaps *PrimaryCaps;
    TDDSCaps *BackBufferCaps;
    TButton *PaintCustom;
    TDDSurfaceDesc *BackBufferDesc;
    TDDPixelFormat *DDPixelFormat1;
    TLabel *Label6;
    TLabel *Label8;
    TEdit *PaletteIndexEdit;
    TEdit *RedEdit;
    TLabel *Label9;
    TListBox *ModesListBox;
    TLabel *Label1;
    TLabel *Label10;
    TLabel *Label13;
    TEdit *GreenEdit;
    TEdit *BlueEdit;
    TLabel *Label11;
    TScrollBar *PaletteIndexScrollBar;
    TScrollBar *RedScrollBar;
    TScrollBar *GreenScrollBar;
    TScrollBar *BlueScrollBar;
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);
    void __fastcall Dx_Draw1EnumDisplayModes(TObject *Sender,
          TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished);
    
    void __fastcall ModesListBoxClick(TObject *Sender);
    void __fastcall PrimarySurfaceCreate(TObject *Sender);
    void __fastcall PrimarySurfaceDestroy(TObject *Sender);
    void __fastcall PrimarySurfaceSurfaceLost(TObject *Sender);
    void __fastcall ModesListBoxDblClick(TObject *Sender);
    
    void __fastcall RepaintButtonClick(TObject *Sender);
    void __fastcall BackBufferCreate(TObject *Sender);
    void __fastcall BackBufferDestroy(TObject *Sender);
    
    void __fastcall PaintWin32Click(TObject *Sender);
    void __fastcall PaintVCLClick(TObject *Sender);
    
    void __fastcall SnapshotButtonClick(TObject *Sender);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
    
    
    void __fastcall PaintCustomClick(TObject *Sender);
    
    
    void __fastcall PaletteIndexScrollBarChange(TObject *Sender);
    void __fastcall RedScrollBarChange(TObject *Sender);
    void __fastcall GreenScrollBarChange(TObject *Sender);
    void __fastcall BlueScrollBarChange(TObject *Sender);
    
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

    void __fastcall VCLPaintFrame(TObject *Sender, bool& Done);
    void __fastcall Win32PaintFrame(TObject *Sender, bool& Done);
    void __fastcall CustomPaintFrame(TObject *Sender, bool& Done);

    void __fastcall OnSurfaceLost(TObject *Sender);

    #define MAX_MODES 50

    int         Widths[MAX_MODES];
    int         Heights[MAX_MODES];
    int         BitDepths[MAX_MODES];
    int         RefreshRates[MAX_MODES];
    int         NumModes;

    int         CurrentPageNumber;
    TCanvas*    canvas;

    bool        CustomPaintPixel(int x1,int y1,dword color);
    bool        CustomPaintLine(int x1,int y1,int x2,int y2,dword color);

    char*       Buffer_Ptr;
    int         Buffer_Pitch;
    int         Buffer_Width;
    int         Buffer_Height;
    int         Buffer_Bits;
};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

